@@ -754,9 +754,9 @@ namespace Jazz2::Compatibility
 		Add(JJ2Event::GEM_BLUE, ConstantParamList(EventType::Gem, { 2 }));
 		Add(JJ2Event::GEM_PURPLE, ConstantParamList(EventType::Gem, { 3 }));
 
-		Add(JJ2Event::GEM_RED_RECT, ConstantParamList(EventType::Gem, { 0 }));
-		Add(JJ2Event::GEM_GREEN_RECT, ConstantParamList(EventType::Gem, { 1 }));
-		Add(JJ2Event::GEM_BLUE_RECT, ConstantParamList(EventType::Gem, { 2 }));
+		Add(JJ2Event::GEM_RED_RECT, ConstantParamList(EventType::Gem, { 0, 2 }));
+		Add(JJ2Event::GEM_GREEN_RECT, ConstantParamList(EventType::Gem, { 1, 2 }));
+		Add(JJ2Event::GEM_BLUE_RECT, ConstantParamList(EventType::Gem, { 2, 2 }));
 
 		Add(JJ2Event::GEM_SUPER, NoParamList(EventType::GemGiant));
 		Add(JJ2Event::GEM_RING, [](JJ2Level* level, std::uint32_t jj2Params) -> ConversionResult {
@@ -853,9 +853,9 @@ namespace Jazz2::Compatibility
 		Add(JJ2Event::CRATE_AMMO_SEEKER, GetAmmoCrateConverter(3));
 		Add(JJ2Event::CRATE_AMMO_RF, GetAmmoCrateConverter(4));
 		Add(JJ2Event::CRATE_AMMO_TOASTER, GetAmmoCrateConverter(5));
-		Add(JJ2Event::CRATE_CARROT, ConstantParamList(EventType::Crate, { (std::uint8_t)EventType::Carrot, (std::uint8_t)((std::int32_t)EventType::Carrot >> 8), 1 }));
-		Add(JJ2Event::CRATE_SPRING, ConstantParamList(EventType::Crate, { (std::uint8_t)EventType::Spring, (std::uint8_t)((std::int32_t)EventType::Spring >> 8), 1, 1 }));
-		Add(JJ2Event::CRATE_ONEUP, ConstantParamList(EventType::Crate, { (std::uint8_t)EventType::OneUp, (std::uint8_t)((std::int32_t)EventType::OneUp >> 8), 1 }));
+		Add(JJ2Event::CRATE_CARROT, ConstantParamList(EventType::Crate, { (std::uint8_t)EventType::Carrot, (std::uint8_t)((std::int32_t)EventType::Carrot >> 8), 1/*Count*/ }));
+		Add(JJ2Event::CRATE_SPRING, ConstantParamList(EventType::Crate, { (std::uint8_t)EventType::Spring, (std::uint8_t)((std::int32_t)EventType::Spring >> 8), 1/*Count*/, 1/*Green*/, 0/*Up*/, 0x01 }));
+		Add(JJ2Event::CRATE_ONEUP, ConstantParamList(EventType::Crate, { (std::uint8_t)EventType::OneUp, (std::uint8_t)((std::int32_t)EventType::OneUp >> 8), 1/*Count*/ }));
 		Add(JJ2Event::CRATE_BOMB, [](JJ2Level* level, std::uint32_t jj2Params) -> ConversionResult {
 			std::uint8_t eventParams[16];
 			ConvertParamInt(jj2Params, {
@@ -965,8 +965,10 @@ namespace Jazz2::Compatibility
 			std::uint8_t flags = 0;
 			if (applyGravitation) flags |= 0x01;
 			if (frozen) flags |= 0x02;
+			if (eventParams[1] != 0) flags |= 0x04;
+			if (eventParams[2] != 0) flags |= 0x08;
 
-			return { EventType::Spring, { type, orientation, eventParams[1], eventParams[2], eventParams[3], flags } };
+			return { EventType::Spring, { type, orientation, flags, eventParams[3] } };
 		};
 	}
 
