@@ -1,13 +1,10 @@
 ﻿#include "LevelHandler.h"
 #include "PlayerViewport.h"
 #include "PreferencesCache.h"
+#include "UI/DiscordRpcClient.h"
 #include "UI/HUD.h"
 #include "../Common.h"
 
-#if (defined(DEATH_TARGET_WINDOWS) && !defined(DEATH_TARGET_WINDOWS_RT)) || defined(DEATH_TARGET_UNIX)
-#	include "UI/DiscordRpcClient.h"
-#endif
-
 #if defined(WITH_ANGELSCRIPT)
 #	include "Scripting/LevelScriptLoader.h"
 #endif
@@ -108,10 +105,12 @@ namespace Jazz2
 #endif
 
 	LevelHandler::LevelHandler(IRootController* root)
-		: _root(root), _lightingShader(nullptr), _blurShader(nullptr), _downsampleShader(nullptr), _combineShader(nullptr), _combineWithWaterShader(nullptr),
-			_eventSpawner(this), _difficulty(GameDifficulty::Default), _isReforged(false), _cheatsUsed(false), _checkpointCreated(false),
-			_cheatsBufferLength(0), _nextLevelType(ExitType::None), _nextLevelTime(0.0f), _elapsedFrames(0.0f), _checkpointFrames(0.0f),
-			_waterLevel(FLT_MAX), _weatherType(WeatherType::None), _pressedKeys(ValueInit, (std::size_t)KeySym::COUNT), _overrideActions(0)
+		: _root(root), _lightingShader(nullptr), _blurShader(nullptr), _downsampleShader(nullptr), _combineShader(nullptr),
+			_combineWithWaterShader(nullptr), _eventSpawner(this), _difficulty(GameDifficulty::Default), _isReforged(false),
+			_cheatsUsed(false), _checkpointCreated(false), _cheatsBufferLength(0), _nextLevelType(ExitType::None),
+			_nextLevelTime(0.0f), _elapsedMillisecondsBegin(0), _elapsedFrames(0.0f), _checkpointFrames(0.0f),
+			_waterLevel(FLT_MAX), _weatherType(WeatherType::None), _pressedKeys(ValueInit, (std::size_t)KeySym::COUNT),
+			_overrideActions(0)
 	{
 	}
 
@@ -135,6 +134,7 @@ namespace Jazz2
 		_difficulty = levelInit.Difficulty;
 		_isReforged = levelInit.IsReforged;
 		_cheatsUsed = levelInit.CheatsUsed;
+		_elapsedMillisecondsBegin = levelInit.ElapsedMilliseconds;
 
 		auto& resolver = ContentResolver::Get();
 		resolver.BeginLoading();
@@ -163,7 +163,7 @@ namespace Jazz2
 		return true;
 	}
 
-	bool LevelHandler::Initialize(Stream& src)
+	bool LevelHandler::Initialize(Stream& src, std::uint16_t version)
 	{
 		ZoneScopedC(0x4876AF);
 
@@ -180,6 +180,9 @@ namespace Jazz2
 		_difficulty = (GameDifficulty)src.ReadValue<std::uint8_t>();
 		_isReforged = (flags & 0x01) != 0;
 		_cheatsUsed = (flags & 0x02) != 0;
+		if (version >= 3) {
+			_elapsedMillisecondsBegin = src.ReadVariableUint64();
+		}
 		_checkpointFrames = src.ReadValue<float>();
 
 		auto& resolver = ContentResolver::Get();
@@ -211,7 +214,7 @@ namespace Jazz2
 
 		for (std::uint32_t i = 0; i < playerCount; i++) {
 			std::shared_ptr<Actors::Player> player = std::make_shared<Actors::Player>();
-			player->InitializeFromStream(this, src);
+			player->InitializeFromStream(this, src, version);
 
 			Actors::Player* ptr = player.get();
 			_players.push_back(ptr);
@@ -324,6 +327,16 @@ namespace Jazz2
 		return _defaultAmbientLight.W;
 	}
 
+	float LevelHandler::GetAmbientLight(Actors::Player* player) const
+	{
+		for (auto& viewport : _assignedViewports) {
+			if (viewport->_targetPlayer == player) {
+				return viewport->_ambientLightTarget;
+			}
+		}
+		return 0.0f;
+	}
+
 	void LevelHandler::SetAmbientLight(Actors::Player* player, float value)
 	{
 		for (auto& viewport : _assignedViewports) {
@@ -389,12 +402,14 @@ namespace Jazz2
 		if (PreferencesCache::AllowUnsignedScripts) {
 			const StringView foundDot = descriptor.FullPath.findLastOr('.', descriptor.FullPath.end());
 			String scriptPath = (foundDot == descriptor.FullPath.end() ? StringView(descriptor.FullPath) : descriptor.FullPath.prefix(foundDot.begin())) + ".j2as"_s;
-			if (fs::IsReadableFile(scriptPath)) {
+			if (auto scriptPathCaseInsensitive = fs::FindPathCaseInsensitive(scriptPath)) {
+				if (fs::IsReadableFile(scriptPathCaseInsensitive)) {
 #	if defined(WITH_ANGELSCRIPT)
-				_scripts = std::make_unique<Scripting::LevelScriptLoader>(this, scriptPath);
+					_scripts = std::make_unique<Scripting::LevelScriptLoader>(this, scriptPathCaseInsensitive);
 #	else
-				LOGW("Level requires scripting, but scripting support is disabled in this build");
+					LOGW("Level requires scripting, but scripting support is disabled in this build");
 #	endif
+				}
 			}
 		}
 #endif
@@ -402,13 +417,7 @@ namespace Jazz2
 
 	void LevelHandler::SpawnPlayers(const LevelInitialization& levelInit)
 	{
-		std::int32_t playerCount = 0;
-		for (std::int32_t i = 0; i < static_cast<std::int32_t>(arraySize(levelInit.PlayerCarryOvers)); i++) {
-			if (levelInit.PlayerCarryOvers[i].Type == PlayerType::None) {
-				continue;
-			}
-			playerCount++;
-		}
+		std::int32_t playerCount = levelInit.GetPlayerCount();
 
 		for (std::int32_t i = 0; i < static_cast<std::int32_t>(arraySize(levelInit.PlayerCarryOvers)); i++) {
 			if (levelInit.PlayerCarryOvers[i].Type == PlayerType::None) {
@@ -440,6 +449,11 @@ namespace Jazz2
 		}
 	}
 
+	Vector2i LevelHandler::GetViewSize() const
+	{
+		return _viewSize;
+	}
+
 	void LevelHandler::OnBeginFrame()
 	{
 		ZoneScopedC(0x4876AF);
@@ -661,109 +675,113 @@ namespace Jazz2
 	{
 		_pressedKeys.set((std::size_t)event.sym);
 
-		// Cheats
-		if (PreferencesCache::AllowCheats && _difficulty != GameDifficulty::Multiplayer && !_players.empty()) {
-			if (event.sym >= KeySym::A && event.sym <= KeySym::Z) {
-				if (event.sym == KeySym::J && _cheatsBufferLength >= 2) {
-					_cheatsBufferLength = 0;
-					_cheatsBuffer[_cheatsBufferLength++] = (char)event.sym;
-				} else if (_cheatsBufferLength < static_cast<std::int32_t>(arraySize(_cheatsBuffer))) {
-					_cheatsBuffer[_cheatsBufferLength++] = (char)event.sym;
+		if (_pauseMenu != nullptr) {
+			_pauseMenu->OnKeyReleased(event);
+		} else {
+			// Cheats
+			if (PreferencesCache::AllowCheats && _difficulty != GameDifficulty::Multiplayer && !_players.empty()) {
+				if (event.sym >= KeySym::A && event.sym <= KeySym::Z) {
+					if (event.sym == KeySym::J && _cheatsBufferLength >= 2) {
+						_cheatsBufferLength = 0;
+						_cheatsBuffer[_cheatsBufferLength++] = (char)event.sym;
+					} else if (_cheatsBufferLength < static_cast<std::int32_t>(arraySize(_cheatsBuffer))) {
+						_cheatsBuffer[_cheatsBufferLength++] = (char)event.sym;
 
-					if (_cheatsBufferLength >= 3 && _cheatsBuffer[0] == (char)KeySym::J && _cheatsBuffer[1] == (char)KeySym::J) {
-						switch (_cheatsBufferLength) {
-							case 3:
-								if (_cheatsBuffer[2] == (char)KeySym::K) {
-									_cheatsBufferLength = 0;
-									_cheatsUsed = true;
-									for (auto* player : _players) {
-										player->TakeDamage(INT32_MAX);
-									}
-								}
-								break;
-							case 5:
-								if (_cheatsBuffer[2] == (char)KeySym::G && _cheatsBuffer[3] == (char)KeySym::O && _cheatsBuffer[4] == (char)KeySym::D) {
-									_cheatsBufferLength = 0;
-									_cheatsUsed = true;
-									for (auto* player : _players) {
-										player->SetInvulnerability(36000.0f, true);
-									}
-								}
-								break;
-							case 6:
-								if (_cheatsBuffer[2] == (char)KeySym::N && _cheatsBuffer[3] == (char)KeySym::E && _cheatsBuffer[4] == (char)KeySym::X && _cheatsBuffer[5] == (char)KeySym::T) {
-									_cheatsBufferLength = 0;
-									_cheatsUsed = true;
-									BeginLevelChange(nullptr, ExitType::Warp | ExitType::FastTransition);
-								} else if ((_cheatsBuffer[2] == (char)KeySym::G && _cheatsBuffer[3] == (char)KeySym::U && _cheatsBuffer[4] == (char)KeySym::N && _cheatsBuffer[5] == (char)KeySym::S) ||
-										   (_cheatsBuffer[2] == (char)KeySym::A && _cheatsBuffer[3] == (char)KeySym::M && _cheatsBuffer[4] == (char)KeySym::M && _cheatsBuffer[5] == (char)KeySym::O)) {
-									_cheatsBufferLength = 0;
-									_cheatsUsed = true;
-									for (auto* player : _players) {
-										for (std::int32_t i = 0; i < (std::int32_t)WeaponType::Count; i++) {
-											player->AddAmmo((WeaponType)i, 99);
+						if (_cheatsBufferLength >= 3 && _cheatsBuffer[0] == (char)KeySym::J && _cheatsBuffer[1] == (char)KeySym::J) {
+							switch (_cheatsBufferLength) {
+								case 3:
+									if (_cheatsBuffer[2] == (char)KeySym::K) {
+										_cheatsBufferLength = 0;
+										_cheatsUsed = true;
+										for (auto* player : _players) {
+											player->TakeDamage(INT32_MAX);
 										}
 									}
-								} else if (_cheatsBuffer[2] == (char)KeySym::R && _cheatsBuffer[3] == (char)KeySym::U && _cheatsBuffer[4] == (char)KeySym::S && _cheatsBuffer[5] == (char)KeySym::H) {
-									_cheatsBufferLength = 0;
-									_cheatsUsed = true;
-									for (auto* player : _players) {
-										player->ActivateSugarRush(1300.0f);
-									}
-								} else if (_cheatsBuffer[2] == (char)KeySym::G && _cheatsBuffer[3] == (char)KeySym::E && _cheatsBuffer[4] == (char)KeySym::M && _cheatsBuffer[5] == (char)KeySym::S) {
-									_cheatsBufferLength = 0;
-									_cheatsUsed = true;
-									for (auto* player : _players) {
-										player->AddGems(5);
-									}
-								} else if (_cheatsBuffer[2] == (char)KeySym::B && _cheatsBuffer[3] == (char)KeySym::I && _cheatsBuffer[4] == (char)KeySym::R && _cheatsBuffer[5] == (char)KeySym::D) {
-									_cheatsBufferLength = 0;
-									_cheatsUsed = true;
-									for (auto* player : _players) {
-										player->SpawnBird(0, player->GetPos());
-									}
-								}
-								break;
-							case 7:
-								if (_cheatsBuffer[2] == (char)KeySym::P && _cheatsBuffer[3] == (char)KeySym::O && _cheatsBuffer[4] == (char)KeySym::W && _cheatsBuffer[5] == (char)KeySym::E && _cheatsBuffer[6] == (char)KeySym::R) {
-									_cheatsBufferLength = 0;
-									_cheatsUsed = true;
-									for (auto* player : _players) {
-										for (std::int32_t i = 0; i < (std::int32_t)WeaponType::Count; i++) {
-											player->AddWeaponUpgrade((WeaponType)i, 0x01);
+									break;
+								case 5:
+									if (_cheatsBuffer[2] == (char)KeySym::G && _cheatsBuffer[3] == (char)KeySym::O && _cheatsBuffer[4] == (char)KeySym::D) {
+										_cheatsBufferLength = 0;
+										_cheatsUsed = true;
+										for (auto* player : _players) {
+											player->SetInvulnerability(36000.0f, Actors::Player::InvulnerableType::Shielded);
 										}
 									}
-								} else if (_cheatsBuffer[2] == (char)KeySym::C && _cheatsBuffer[3] == (char)KeySym::O && _cheatsBuffer[4] == (char)KeySym::I && _cheatsBuffer[5] == (char)KeySym::N && _cheatsBuffer[6] == (char)KeySym::S) {
-									_cheatsBufferLength = 0;
-									_cheatsUsed = true;
-									// Coins are synchronized automatically
-									_players[0]->AddCoins(5);
-								} else if (_cheatsBuffer[2] == (char)KeySym::M && _cheatsBuffer[3] == (char)KeySym::O && _cheatsBuffer[4] == (char)KeySym::R && _cheatsBuffer[5] == (char)KeySym::P && _cheatsBuffer[6] == (char)KeySym::H) {
-									_cheatsBufferLength = 0;
-									_cheatsUsed = true;
+									break;
+								case 6:
+									if (_cheatsBuffer[2] == (char)KeySym::N && _cheatsBuffer[3] == (char)KeySym::E && _cheatsBuffer[4] == (char)KeySym::X && _cheatsBuffer[5] == (char)KeySym::T) {
+										_cheatsBufferLength = 0;
+										_cheatsUsed = true;
+										BeginLevelChange(nullptr, ExitType::Warp | ExitType::FastTransition);
+									} else if ((_cheatsBuffer[2] == (char)KeySym::G && _cheatsBuffer[3] == (char)KeySym::U && _cheatsBuffer[4] == (char)KeySym::N && _cheatsBuffer[5] == (char)KeySym::S) ||
+											   (_cheatsBuffer[2] == (char)KeySym::A && _cheatsBuffer[3] == (char)KeySym::M && _cheatsBuffer[4] == (char)KeySym::M && _cheatsBuffer[5] == (char)KeySym::O)) {
+										_cheatsBufferLength = 0;
+										_cheatsUsed = true;
+										for (auto* player : _players) {
+											for (std::int32_t i = 0; i < (std::int32_t)WeaponType::Count; i++) {
+												player->AddAmmo((WeaponType)i, 99);
+											}
+										}
+									} else if (_cheatsBuffer[2] == (char)KeySym::R && _cheatsBuffer[3] == (char)KeySym::U && _cheatsBuffer[4] == (char)KeySym::S && _cheatsBuffer[5] == (char)KeySym::H) {
+										_cheatsBufferLength = 0;
+										_cheatsUsed = true;
+										for (auto* player : _players) {
+											player->ActivateSugarRush(1300.0f);
+										}
+									} else if (_cheatsBuffer[2] == (char)KeySym::G && _cheatsBuffer[3] == (char)KeySym::E && _cheatsBuffer[4] == (char)KeySym::M && _cheatsBuffer[5] == (char)KeySym::S) {
+										_cheatsBufferLength = 0;
+										_cheatsUsed = true;
+										for (auto* player : _players) {
+											player->AddGems(0, 5);
+										}
+									} else if (_cheatsBuffer[2] == (char)KeySym::B && _cheatsBuffer[3] == (char)KeySym::I && _cheatsBuffer[4] == (char)KeySym::R && _cheatsBuffer[5] == (char)KeySym::D) {
+										_cheatsBufferLength = 0;
+										_cheatsUsed = true;
+										for (auto* player : _players) {
+											player->SpawnBird(0, player->GetPos());
+										}
+									}
+									break;
+								case 7:
+									if (_cheatsBuffer[2] == (char)KeySym::P && _cheatsBuffer[3] == (char)KeySym::O && _cheatsBuffer[4] == (char)KeySym::W && _cheatsBuffer[5] == (char)KeySym::E && _cheatsBuffer[6] == (char)KeySym::R) {
+										_cheatsBufferLength = 0;
+										_cheatsUsed = true;
+										for (auto* player : _players) {
+											for (std::int32_t i = 0; i < (std::int32_t)WeaponType::Count; i++) {
+												player->AddWeaponUpgrade((WeaponType)i, 0x01);
+											}
+										}
+									} else if (_cheatsBuffer[2] == (char)KeySym::C && _cheatsBuffer[3] == (char)KeySym::O && _cheatsBuffer[4] == (char)KeySym::I && _cheatsBuffer[5] == (char)KeySym::N && _cheatsBuffer[6] == (char)KeySym::S) {
+										_cheatsBufferLength = 0;
+										_cheatsUsed = true;
+										// Coins are synchronized automatically
+										_players[0]->AddCoins(5);
+									} else if (_cheatsBuffer[2] == (char)KeySym::M && _cheatsBuffer[3] == (char)KeySym::O && _cheatsBuffer[4] == (char)KeySym::R && _cheatsBuffer[5] == (char)KeySym::P && _cheatsBuffer[6] == (char)KeySym::H) {
+										_cheatsBufferLength = 0;
+										_cheatsUsed = true;
 
-									PlayerType newType;
-									switch (_players[0]->GetPlayerType()) {
-										case PlayerType::Jazz: newType = PlayerType::Spaz; break;
-										case PlayerType::Spaz: newType = PlayerType::Lori; break;
-										default: newType = PlayerType::Jazz; break;
-									}
+										PlayerType newType;
+										switch (_players[0]->GetPlayerType()) {
+											case PlayerType::Jazz: newType = PlayerType::Spaz; break;
+											case PlayerType::Spaz: newType = PlayerType::Lori; break;
+											default: newType = PlayerType::Jazz; break;
+										}
 
-									if (!_players[0]->MorphTo(newType)) {
-										_players[0]->MorphTo(PlayerType::Jazz);
+										if (!_players[0]->MorphTo(newType)) {
+											_players[0]->MorphTo(PlayerType::Jazz);
+										}
 									}
-								}
-								break;
-							case 8:
-								if (_cheatsBuffer[2] == (char)KeySym::S && _cheatsBuffer[3] == (char)KeySym::H && _cheatsBuffer[4] == (char)KeySym::I && _cheatsBuffer[5] == (char)KeySym::E && _cheatsBuffer[6] == (char)KeySym::L && _cheatsBuffer[7] == (char)KeySym::D) {
-									_cheatsBufferLength = 0;
-									_cheatsUsed = true;
-									for (auto* player : _players) {
-										ShieldType shieldType = (ShieldType)(((std::int32_t)player->GetActiveShield() + 1) % (std::int32_t)ShieldType::Count);
-										player->SetShield(shieldType, 40.0f * FrameTimer::FramesPerSecond);
+									break;
+								case 8:
+									if (_cheatsBuffer[2] == (char)KeySym::S && _cheatsBuffer[3] == (char)KeySym::H && _cheatsBuffer[4] == (char)KeySym::I && _cheatsBuffer[5] == (char)KeySym::E && _cheatsBuffer[6] == (char)KeySym::L && _cheatsBuffer[7] == (char)KeySym::D) {
+										_cheatsBufferLength = 0;
+										_cheatsUsed = true;
+										for (auto* player : _players) {
+											ShieldType shieldType = (ShieldType)(((std::int32_t)player->GetActiveShield() + 1) % (std::int32_t)ShieldType::Count);
+											player->SetShield(shieldType, 40.0f * FrameTimer::FramesPerSecond);
+										}
 									}
-								}
-								break;
+									break;
+							}
 						}
 					}
 				}
@@ -774,6 +792,10 @@ namespace Jazz2
 	void LevelHandler::OnKeyReleased(const KeyboardEvent& event)
 	{
 		_pressedKeys.reset((std::size_t)event.sym);
+
+		if (_pauseMenu != nullptr) {
+			_pauseMenu->OnKeyReleased(event);
+		}
 	}
 
 	void LevelHandler::OnTouchEvent(const  TouchEvent& event)
@@ -923,7 +945,7 @@ namespace Jazz2
 		return (*collider == nullptr);
 	}
 
-	void LevelHandler::FindCollisionActorsByAABB(Actors::ActorBase* self, const AABBf& aabb, Function<bool(Actors::ActorBase*)>&& callback)
+	void LevelHandler::FindCollisionActorsByAABB(const Actors::ActorBase* self, const AABBf& aabb, Function<bool(Actors::ActorBase*)>&& callback)
 	{
 		struct QueryHelper {
 			const LevelHandler* Handler;
@@ -1089,12 +1111,20 @@ namespace Jazz2
 
 	void LevelHandler::HandleGameOver(Actors::Player* player)
 	{
-		// TODO: Implement Game Over screen
-		_root->GoToMainMenu(false);
+		LevelInitialization levelInit;
+		PrepareNextLevelInitialization(levelInit);
+		levelInit.LevelName = ":gameover"_s;
+		_root->ChangeLevel(std::move(levelInit));
 	}
 
-	bool LevelHandler::HandlePlayerDied(Actors::Player* player)
+	bool LevelHandler::HandlePlayerDied(Actors::Player* player, Actors::ActorBase* collider)
 	{
+#if defined(WITH_ANGELSCRIPT)
+		if (_scripts != nullptr) {
+			_scripts->OnPlayerDied(player, collider);
+		}
+#endif
+
 		if (_activeBoss != nullptr) {
 			if (_activeBoss->OnPlayerDied()) {
 				_activeBoss = nullptr;
@@ -1145,12 +1175,12 @@ namespace Jazz2
 		}
 	}
 
-	void LevelHandler::HandlePlayerGems(Actors::Player* player, std::int32_t prevCount, std::int32_t newCount)
+	void LevelHandler::HandlePlayerGems(Actors::Player* player, std::uint8_t gemType, std::int32_t prevCount, std::int32_t newCount)
 	{
 		// Show notification only for local players (which have assigned viewport)
 		for (auto& viewport : _assignedViewports) {
 			if (viewport->_targetPlayer == player) {
-				_hud->ShowGems(newCount);
+				_hud->ShowGems(gemType, newCount);
 				break;
 			}
 		}
@@ -1164,7 +1194,7 @@ namespace Jazz2
 		float ambientLight = _defaultAmbientLight.W;
 		for (auto& viewport : _assignedViewports) {
 			if (viewport->_targetPlayer == player) {
-				viewport->_ambientLightTarget;
+				ambientLight = viewport->_ambientLightTarget;
 				break;
 			}
 		}
@@ -1372,6 +1402,7 @@ namespace Jazz2
 		dest.Write(_levelFileName.data(), (std::uint32_t)_levelFileName.size());
 
 		dest.WriteValue<std::uint8_t>((std::uint8_t)_difficulty);
+		dest.WriteVariableUint64(_elapsedMillisecondsBegin);
 		dest.WriteValue<float>(_checkpointFrames);
 		dest.WriteValue<float>(_waterLevel);
 		dest.WriteValue<std::uint8_t>((std::uint8_t)_weatherType);
@@ -1417,11 +1448,6 @@ namespace Jazz2
 		}
 	}
 
-	Vector2i LevelHandler::GetViewSize() const
-	{
-		return _viewSize;
-	}
-
 	void LevelHandler::BeforeActorDestroyed(Actors::ActorBase* actor)
 	{
 		// Nothing to do here
@@ -1526,6 +1552,7 @@ namespace Jazz2
 		levelInit.CheatsUsed = _cheatsUsed;
 		levelInit.LastExitType = _nextLevelType;
 		levelInit.LastEpisodeName = _episodeName;
+		levelInit.ElapsedMilliseconds = _elapsedMillisecondsBegin + (std::uint64_t)(_elapsedFrames * FrameTimer::SecondsPerFrame * 1000.0f);
 
 		for (std::int32_t i = 0; i < _players.size(); i++) {
 			levelInit.PlayerCarryOvers[i] = _players[i]->PrepareLevelCarryOver();
@@ -1775,6 +1802,16 @@ namespace Jazz2
 		viewport._camera->setView(viewport._cameraPos, 0.0f, 1.0f);
 	}
 
+	Vector2f LevelHandler::GetCameraPos(Actors::Player* player) const
+	{
+		for (auto& viewport : _assignedViewports) {
+			if (viewport->_targetPlayer == player) {
+				return viewport->_cameraPos;
+			}
+		}
+		return {};
+	}
+
 	void LevelHandler::LimitCameraView(Actors::Player* player, std::int32_t left, std::int32_t width)
 	{
 		_levelBounds.X = left;
@@ -1842,6 +1879,15 @@ namespace Jazz2
 		}
 	}
 
+	void LevelHandler::OverrideCameraView(Actors::Player* player, float x, float y, bool topLeft)
+	{
+		for (auto& viewport : _assignedViewports) {
+			if (viewport->_targetPlayer == player) {
+				viewport->OverrideCamera(x, y, topLeft);
+			}
+		}
+	}
+
 	void LevelHandler::ShakeCameraView(Actors::Player* player, float duration)
 	{
 		for (auto& viewport : _assignedViewports) {
@@ -1972,7 +2018,7 @@ namespace Jazz2
 
 	void LevelHandler::UpdateRichPresence()
 	{
-#if (defined(DEATH_TARGET_WINDOWS) && !defined(DEATH_TARGET_WINDOWS_RT)) || defined(DEATH_TARGET_UNIX)
+#if (defined(DEATH_TARGET_WINDOWS) && !defined(DEATH_TARGET_WINDOWS_RT)) || (defined(DEATH_TARGET_UNIX) && !defined(DEATH_TARGET_SWITCH))
 		if (!PreferencesCache::EnableDiscordIntegration || !UI::DiscordRpcClient::Get().IsSupported()) {
 			return;
 		}
