@@ -556,7 +556,7 @@ namespace Jazz2::Multiplayer
 		return LevelHandler::IsPositionEmpty(self, aabb, params, collider);
 	}
 
-	void MultiLevelHandler::FindCollisionActorsByAABB(Actors::ActorBase* self, const AABBf& aabb, Function<bool(Actors::ActorBase*)>&& callback)
+	void MultiLevelHandler::FindCollisionActorsByAABB(const Actors::ActorBase* self, const AABBf& aabb, Function<bool(Actors::ActorBase*)>&& callback)
 	{
 		LevelHandler::FindCollisionActorsByAABB(self, aabb, std::move(callback));
 	}
@@ -594,10 +594,10 @@ namespace Jazz2::Multiplayer
 		//LevelHandler::HandleGameOver(player);
 	}
 
-	bool MultiLevelHandler::HandlePlayerDied(Actors::Player* player)
+	bool MultiLevelHandler::HandlePlayerDied(Actors::Player* player, Actors::ActorBase* collider)
 	{
 		// TODO: Remove this override
-		return LevelHandler::HandlePlayerDied(player);
+		return LevelHandler::HandlePlayerDied(player, collider);
 	}
 
 	void MultiLevelHandler::HandlePlayerLevelChanging(Actors::Player* player, ExitType exitType)
@@ -810,9 +810,9 @@ namespace Jazz2::Multiplayer
 		}
 	}
 
-	void MultiLevelHandler::HandlePlayerGems(Actors::Player* player, std::int32_t prevCount, std::int32_t newCount)
+	void MultiLevelHandler::HandlePlayerGems(Actors::Player* player, std::uint8_t gemType, std::int32_t prevCount, std::int32_t newCount)
 	{
-		LevelHandler::HandlePlayerGems(player, prevCount, newCount);
+		LevelHandler::HandlePlayerGems(player, gemType, prevCount, newCount);
 
 		if (_isServer) {
 			for (const auto& [peer, peerDesc] : _peerDesc) {
@@ -820,6 +820,7 @@ namespace Jazz2::Multiplayer
 					MemoryStream packet(9);
 					packet.WriteValue<std::uint8_t>((std::uint8_t)ServerPacketType::PlayerRefreshGems);
 					packet.WriteVariableUint32(player->_playerIndex);
+					packet.WriteValue<std::uint8_t>(gemType);
 					packet.WriteVariableInt32(newCount);
 					_networkManager->SendToPeer(peer, NetworkChannel::Main, packet);
 					break;
@@ -1317,7 +1318,7 @@ namespace Jazz2::Multiplayer
 							// TODO: gain, pitch, ...
 							it->second->PlaySfx(identifier, gain, pitch);
 						}
-					});
+					}, NCINE_CURRENT_FUNCTION);
 					break;
 				}
 				case ServerPacketType::PlayCommonSfx: {
@@ -1333,7 +1334,7 @@ namespace Jazz2::Multiplayer
 					// TODO: Use only lock here
 					_root->InvokeAsync([this, posX, posY, gain, pitch, identifier = std::move(identifier)]() {
 						PlayCommonSfx(identifier, Vector3f((float)posX, (float)posY, 0.0f), gain, pitch);
-					});
+					}, NCINE_CURRENT_FUNCTION);
 					break;
 				}
 				case ServerPacketType::ShowMessage: {
@@ -1380,7 +1381,7 @@ namespace Jazz2::Multiplayer
 						// TODO: Needed to initialize newly assigned viewport, because it called asynchronously, not from handler initialization
 						Vector2i res = theApplication().GetResolution();
 						OnInitializeViewport(res.X, res.Y);
-					});
+					}, NCINE_CURRENT_FUNCTION);
 					return true;
 				}
 				case ServerPacketType::CreateRemoteActor: {
@@ -1405,7 +1406,7 @@ namespace Jazz2::Multiplayer
 
 						_remoteActors[actorId] = remoteActor;
 						AddActor(std::static_pointer_cast<Actors::ActorBase>(remoteActor));
-					});
+					}, NCINE_CURRENT_FUNCTION);
 					return true;
 				}
 				case ServerPacketType::CreateMirroredActor: {
@@ -1431,7 +1432,7 @@ namespace Jazz2::Multiplayer
 						} else {
 							LOGD("[MP] ServerPacketType::CreateMirroredActor - CANNOT CREATE - actorId: %u", actorId);
 						}
-					});
+					}, NCINE_CURRENT_FUNCTION);
 					return true;
 				}
 				case ServerPacketType::DestroyRemoteActor: {
@@ -1449,7 +1450,7 @@ namespace Jazz2::Multiplayer
 						} else {
 							LOGD("[MP] ServerPacketType::DestroyRemoteActor - NOT FOUND - actorId: %u", actorId);
 						}
-					});
+					}, NCINE_CURRENT_FUNCTION);
 					return true;
 				}
 				case ServerPacketType::UpdateAllActors: {
@@ -1493,7 +1494,7 @@ namespace Jazz2::Multiplayer
 
 					_root->InvokeAsync([this, triggerId, newState]() {
 						TileMap()->SetTrigger(triggerId, newState);
-					});
+					}, NCINE_CURRENT_FUNCTION);
 					return true;
 				}
 				case ServerPacketType::AdvanceTileAnimation: {
@@ -1506,7 +1507,7 @@ namespace Jazz2::Multiplayer
 
 					_root->InvokeAsync([this, tx, ty, amount]() {
 						TileMap()->AdvanceDestructibleTileAnimation(tx, ty, amount);
-					});
+					}, NCINE_CURRENT_FUNCTION);
 					return true;
 				}
 				case ServerPacketType::PlayerMoveInstantly: {
@@ -1526,7 +1527,7 @@ namespace Jazz2::Multiplayer
 
 					_root->InvokeAsync([this, posX, posY, speedX, speedY]() {
 						static_cast<Actors::Multiplayer::RemotablePlayer*>(_players[0])->MoveRemotely(Vector2f(posX, posY), Vector2f(speedX, speedY));
-					});
+					}, NCINE_CURRENT_FUNCTION);
 					return true;
 				}
 				case ServerPacketType::PlayerAckWarped: {
@@ -1625,12 +1626,15 @@ namespace Jazz2::Multiplayer
 						return true;
 					}
 
+					std::uint8_t gemType = packet.ReadValue<std::uint8_t>();
 					std::int32_t newCount = packet.ReadVariableInt32();
 
-					LOGD("[MP] ServerPacketType::PlayerRefreshGems - playerIndex: %u, newCount: %i", playerIndex, newCount);
+					LOGD("[MP] ServerPacketType::PlayerRefreshGems - playerIndex: %u, gemType: %u, newCount: %i", playerIndex, gemType, newCount);
 
-					_players[0]->_gems = newCount;
-					_hud->ShowGems(newCount);
+					if (gemType < arraySize(_players[0]->_gems)) {
+						_players[0]->_gems[gemType] = newCount;
+						_hud->ShowGems(gemType, newCount);
+					}
 					return true;
 				}
 				case ServerPacketType::PlayerTakeDamage: {
@@ -1648,7 +1652,7 @@ namespace Jazz2::Multiplayer
 
 					_root->InvokeAsync([this, health, pushForce]() {
 						_players[0]->TakeDamage(_players[0]->_health - health, pushForce);
-					});
+					}, NCINE_CURRENT_FUNCTION);
 					return true;
 				}
 				case ServerPacketType::PlayerActivateSpring: {
@@ -1671,7 +1675,7 @@ namespace Jazz2::Multiplayer
 							_players[0]->_controllable = true;
 							_players[0]->EndDamagingMove();
 						}
-					});
+					}, NCINE_CURRENT_FUNCTION);
 					return true;
 				}
 				case ServerPacketType::PlayerWarpIn: {
@@ -1687,7 +1691,7 @@ namespace Jazz2::Multiplayer
 
 					_root->InvokeAsync([this, exitType]() {
 						static_cast<Actors::Multiplayer::RemotablePlayer*>(_players[0])->WarpIn(exitType);
-					});
+					}, NCINE_CURRENT_FUNCTION);
 					return true;
 				}
 			}
