@@ -218,8 +218,8 @@ namespace Jazz2
 
 	PlayerViewport::PlayerViewport(LevelHandler* levelHandler, Actors::Player* targetPlayer)
 		: _levelHandler(levelHandler), _targetPlayer(targetPlayer),
-		_downsamplePass(this), _blurPass1(this), _blurPass2(this), _blurPass3(this), _blurPass4(this),
-		_cameraResponsiveness(1.0f, 1.0f), _shakeDuration(0.0f)
+			_downsamplePass(this), _blurPass1(this), _blurPass2(this), _blurPass3(this), _blurPass4(this),
+			_cameraResponsiveness(ResponsivenessMax, ResponsivenessMax), _shakeDuration(0.0f)
 	{
 		_ambientLight = levelHandler->_defaultAmbientLight;
 		_ambientLightTarget = _ambientLight.W;
@@ -323,13 +323,6 @@ namespace Jazz2
 	{
 		ZoneScopedC(0x4876AF);
 
-		constexpr float ResponsivenessChange = 0.04f;
-		constexpr float ResponsivenessMin = 0.3f;
-		constexpr float SlowRatioX = 0.3f;
-		constexpr float SlowRatioY = 0.3f;
-		constexpr float FastRatioX = 0.2f;
-		constexpr float FastRatioY = 0.04f;
-
 		// Ambient Light Transition
 		if (_ambientLight.W != _ambientLightTarget) {
 			float step = timeMult * 0.012f;
@@ -356,39 +349,56 @@ namespace Jazz2
 		Vector2i halfView = _view->size() / 2;
 		Vector2f focusPos = _targetPlayer->GetPos();
 
+		bool overridePosX = false, overridePosY = false;
+		// TODO: Not working correctly on some platforms
+		/*if (!std::isinf(_cameraOverridePos.X)) {
+			overridePosX = true;
+			focusPos.X = _cameraOverridePos.X;
+			if (focusPos.X <= 0.0f) {
+				focusPos.X = halfView.X - focusPos.X;
+			}
+		}
+		if (!std::isinf(_cameraOverridePos.Y)) {
+			overridePosY = true;
+			focusPos.Y = _cameraOverridePos.Y;
+			if (focusPos.Y <= 0.0f) {
+				focusPos.Y = halfView.Y - 1.0f - focusPos.Y;
+			}
+		}*/
+
 		// If player doesn't move but has some speed, it's probably stuck, so reset the speed
 		Vector2f focusSpeed = _targetPlayer->GetSpeed();
-		if (std::abs(_cameraLastPos.X - focusPos.X) < 1.0f) {
+		if (overridePosX || std::abs(_cameraLastPos.X - focusPos.X) < 1.0f) {
 			focusSpeed.X = 0.0f;
 		}
-		if (std::abs(_cameraLastPos.Y - focusPos.Y) < 1.0f) {
+		if (overridePosY || std::abs(_cameraLastPos.Y - focusPos.Y) < 1.0f) {
 			focusSpeed.Y = 0.0f;
 		}
 
 		Vector2f focusVelocity = Vector2f(std::abs(focusSpeed.X), std::abs(focusSpeed.Y));
 
 		// Camera responsiveness (smoothing unexpected movements)
-		if (focusVelocity.X < 1.0f) {
+		if (focusVelocity.X <= 3.0f) {
 			if (_cameraResponsiveness.X > ResponsivenessMin) {
 				_cameraResponsiveness.X = std::max(_cameraResponsiveness.X - ResponsivenessChange * timeMult, ResponsivenessMin);
 			}
 		} else {
-			if (_cameraResponsiveness.X < 1.0f) {
-				_cameraResponsiveness.X = std::min(_cameraResponsiveness.X + ResponsivenessChange * timeMult, 1.0f);
+			if (_cameraResponsiveness.X < ResponsivenessMax) {
+				_cameraResponsiveness.X = std::min(_cameraResponsiveness.X + ResponsivenessChange * timeMult, ResponsivenessMax);
 			}
 		}
-		if (focusVelocity.Y < 1.0f) {
+		if (focusVelocity.Y <= 3.0f) {
 			if (_cameraResponsiveness.Y > ResponsivenessMin) {
 				_cameraResponsiveness.Y = std::max(_cameraResponsiveness.Y - ResponsivenessChange * timeMult, ResponsivenessMin);
 			}
 		} else {
-			if (_cameraResponsiveness.Y < 1.0f) {
-				_cameraResponsiveness.Y = std::min(_cameraResponsiveness.Y + ResponsivenessChange * timeMult, 1.0f);
+			if (_cameraResponsiveness.Y < ResponsivenessMax) {
+				_cameraResponsiveness.Y = std::min(_cameraResponsiveness.Y + ResponsivenessChange * timeMult, ResponsivenessMax);
 			}
 		}
 
-		_cameraLastPos.X = lerpByTime(_cameraLastPos.X, focusPos.X, _cameraResponsiveness.X, timeMult);
-		_cameraLastPos.Y = lerpByTime(_cameraLastPos.Y, focusPos.Y, _cameraResponsiveness.Y, timeMult);
+		_cameraLastPos.X = lerpByTime(_cameraLastPos.X, focusPos.X, std::min(_cameraResponsiveness.X, 1.0f), timeMult);
+		_cameraLastPos.Y = lerpByTime(_cameraLastPos.Y, focusPos.Y, std::min(_cameraResponsiveness.Y, 1.0f), timeMult);
 
 		_cameraDistanceFactor.X = lerpByTime(_cameraDistanceFactor.X, focusSpeed.X * 8.0f, (focusVelocity.X < 2.0f ? SlowRatioX : FastRatioX), timeMult);
 		_cameraDistanceFactor.Y = lerpByTime(_cameraDistanceFactor.Y, focusSpeed.Y * 5.0f, (focusVelocity.Y < 2.0f ? SlowRatioY : FastRatioY), timeMult);
@@ -406,7 +416,9 @@ namespace Jazz2
 		}
 
 		// Clamp camera position to level bounds
-		if (_viewBounds.W > halfView.X * 2) {
+		if (overridePosX) {
+			_cameraPos.X = _cameraLastPos.X + _shakeOffset.X;
+		} else if (_viewBounds.W > halfView.X * 2) {
 			_cameraPos.X = std::clamp(_cameraLastPos.X + _cameraDistanceFactor.X, _viewBounds.X + halfView.X, _viewBounds.X + _viewBounds.W - halfView.X) + _shakeOffset.X;
 			if (!PreferencesCache::UnalignedViewport || std::abs(_cameraDistanceFactor.X) < 1.0f) {
 				_cameraPos.X = std::floor(_cameraPos.X);
@@ -414,7 +426,9 @@ namespace Jazz2
 		} else {
 			_cameraPos.X = std::floor(_viewBounds.X + _viewBounds.W * 0.5f + _shakeOffset.X);
 		}
-		if (_viewBounds.H > halfView.Y * 2) {
+		if (overridePosY) {
+			_cameraPos.Y = _cameraLastPos.Y + _shakeOffset.Y;
+		} else if (_viewBounds.H > halfView.Y * 2) {
 			_cameraPos.Y = std::clamp(_cameraLastPos.Y + _cameraDistanceFactor.Y, _viewBounds.Y + halfView.Y - 1.0f, _viewBounds.Y + _viewBounds.H - halfView.Y - 2.0f) + _shakeOffset.Y;
 			if (!PreferencesCache::UnalignedViewport || std::abs(_cameraDistanceFactor.Y) < 1.0f) {
 				_cameraPos.Y = std::floor(_cameraPos.Y);
@@ -433,6 +447,19 @@ namespace Jazz2
 		}
 	}
 
+	void PlayerViewport::OverrideCamera(float x, float y, bool topLeft)
+	{
+		// TODO: Not working correctly on some platforms
+		/*if (topLeft) {
+			// Use negative values for top-left alignment
+			_cameraOverridePos.X = -x;
+			_cameraOverridePos.Y = -y;
+		} else {
+			_cameraOverridePos.X = x;
+			_cameraOverridePos.Y = y;
+		}*/
+	}
+
 	void PlayerViewport::WarpCameraToTarget(bool fast)
 	{
 		Vector2f focusPos = _targetPlayer->GetPos();
@@ -440,7 +467,7 @@ namespace Jazz2
 			_cameraPos = focusPos;
 			_cameraLastPos = _cameraPos;
 			_cameraDistanceFactor = Vector2f(0.0f, 0.0f);
-			_cameraResponsiveness = Vector2f(1.0f, 1.0f);
+			_cameraResponsiveness = Vector2f(ResponsivenessMax, ResponsivenessMax);
 		} else {
 			Vector2f diff = _cameraLastPos - _cameraPos;
 			_cameraPos = focusPos;
