@@ -26,6 +26,7 @@ namespace Jazz2::UI::Menu
 #else
 			PlayCustomLevels,
 #endif
+			Highscores,
 			Options,
 			About,
 #if !defined(DEATH_TARGET_EMSCRIPTEN) && !defined(DEATH_TARGET_IOS) && !defined(DEATH_TARGET_SWITCH)
@@ -42,13 +43,14 @@ namespace Jazz2::UI::Menu
 			ItemData(Item type, StringView name) : Type(type), Name(name), Y(0.0f) { }
 		};
 
-		static constexpr float DisabledItem = -1024.0f;
-
 		SmallVector<ItemData, (std::int32_t)Item::Count> _items;
 		std::int32_t _selectedIndex;
 		float _animation;
 		float _transitionTime;
+		bool _isPlayable;
+		bool _skipSecondItem;
 		bool _shouldStart;
+		bool _alreadyStarted;
 #if !defined(DEATH_TARGET_EMSCRIPTEN)
 		String _sourcePath;
 #endif
