@@ -16,7 +16,8 @@ using namespace Jazz2::UI::Menu::Resources;
 namespace Jazz2::UI::Menu
 {
 	EpisodeSelectSection::EpisodeSelectSection(bool multiplayer)
-		: _multiplayer(multiplayer), _expandedAnimation(0.0f), _expanded(false), _shouldStart(false)
+		: _multiplayer(multiplayer), _expandedAnimation(0.0f), _transitionFromEpisode(-1), _transitionFromEpisodeTime(0.0f),
+			_expanded(false), _shouldStart(false)
 	{
 		auto& resolver = ContentResolver::Get();
 
@@ -29,6 +30,20 @@ namespace Jazz2::UI::Menu
 			AddEpisode(item);
 		}
 
+		std::int32_t maxPosition = 0;
+		for (const ListViewItem& item : _items) {
+			if (maxPosition < item.Item.Description.Position) {
+				maxPosition = item.Item.Description.Position;
+			}
+		}
+
+		// Move all episodes with Position = 0 to the end of the list
+		for (ListViewItem& item : _items) {
+			if (item.Item.Description.Position == 0) {
+				item.Item.Description.Position = ++maxPosition;
+			}
+		}
+
 		sort(_items.begin(), _items.end(), [](const ListViewItem& a, const ListViewItem& b) -> bool {
 			return (a.Item.Description.Position < b.Item.Description.Position);
 		});
@@ -43,6 +58,12 @@ namespace Jazz2::UI::Menu
 		if (_expanded && _expandedAnimation < 1.0f) {
 			_expandedAnimation = std::min(_expandedAnimation + timeMult * 0.016f, 1.0f);
 		}
+		if (_transitionFromEpisode != -1) {
+			_transitionFromEpisodeTime = lerpByTime(_transitionFromEpisodeTime, 0.0f, 0.07f, timeMult);
+			if (std::abs(_transitionFromEpisodeTime) < 0.01f) {
+				_transitionFromEpisode = -1;
+			}
+		}
 
 		if (!_shouldStart) {
 			if (_root->ActionHit(PlayerActions::Menu)) {
@@ -69,6 +90,8 @@ namespace Jazz2::UI::Menu
 					if (_root->ActionHit(PlayerActions::Up)) {
 						_root->PlaySfx("MenuSelect"_s, 0.5f);
 						_animation = 0.0f;
+						_transitionFromEpisode = _selectedIndex;
+						_transitionFromEpisodeTime = -1.0f;
 
 						_expanded = false;
 						_expandedAnimation = 0.0f;
@@ -82,6 +105,8 @@ namespace Jazz2::UI::Menu
 					} else if (_root->ActionHit(PlayerActions::Down)) {
 						_root->PlaySfx("MenuSelect"_s, 0.5f);
 						_animation = 0.0f;
+						_transitionFromEpisode = _selectedIndex;
+						_transitionFromEpisodeTime = 1.0f;
 
 						_expanded = false;
 						_expandedAnimation = 0.0f;
@@ -236,6 +261,26 @@ namespace Jazz2::UI::Menu
 	void EpisodeSelectSection::OnDrawClipped(Canvas* canvas)
 	{
 		if (!_items.empty()) {
+			bool inTransition = false;
+			if (_transitionFromEpisode != -1 && _transitionFromEpisode != _selectedIndex) {
+				auto& item = _items[_transitionFromEpisode];
+				if (item.Item.Description.BackgroundImage != nullptr) {
+					Vector2f center = Vector2f(canvas->ViewSize.X * 0.5f, canvas->ViewSize.Y * 0.7f);
+					Vector2i backgroundSize = item.Item.Description.BackgroundImage->size();
+
+					float expandedAnimation2 = std::min(_expandedAnimation * 6.0f, 1.0f);
+					float expandedAnimation3 = (expandedAnimation2 * expandedAnimation2 * (3.0f - 2.0f * expandedAnimation2));
+					if (expandedAnimation3 > 0.0f) {
+						backgroundSize += expandedAnimation3 * 100;
+					}
+					
+					inTransition = true;
+					_root->DrawSolid(center.X, center.Y, IMenuContainer::BackgroundLayer - 10, Alignment::Center, Vector2f(backgroundSize.X + 2.0f, backgroundSize.Y + 2.0f), Colorf(1.0f, 1.0f, 1.0f, 0.26f));
+					float alpha = 0.4f - expandedAnimation3 * 0.1f;
+					_root->DrawTexture(*item.Item.Description.BackgroundImage, center.X, center.Y, IMenuContainer::BackgroundLayer - 1, Alignment::Center, Vector2f(backgroundSize.X, backgroundSize.Y), Colorf(alpha, alpha, alpha, 1.0f));
+				}
+			}
+
 			auto& item = _items[_selectedIndex];
 			if (item.Item.Description.BackgroundImage != nullptr) {
 				Vector2f center = Vector2f(canvas->ViewSize.X * 0.5f, canvas->ViewSize.Y * 0.7f);
@@ -247,9 +292,16 @@ namespace Jazz2::UI::Menu
 					backgroundSize += expandedAnimation3 * 100;
 				}
 
-				_root->DrawSolid(center.X, center.Y, IMenuContainer::BackgroundLayer - 10, Alignment::Center, Vector2f(backgroundSize.X + 2.0f, backgroundSize.Y + 2.0f), Colorf(1.0f, 1.0f, 1.0f, 0.26f));
+				float transitionOffset = 0.0f, transitionAlpha = 1.0f;
+				if (!inTransition) {
+					_root->DrawSolid(center.X, center.Y, IMenuContainer::BackgroundLayer - 10, Alignment::Center, Vector2f(backgroundSize.X + 2.0f, backgroundSize.Y + 2.0f), Colorf(1.0f, 1.0f, 1.0f, 0.26f));
+				} else {
+					transitionOffset = 50.0f * _transitionFromEpisodeTime;
+					transitionAlpha = 1.0f - 0.5 * std::abs(_transitionFromEpisodeTime);
+				}
+				
 				float alpha = 0.4f - expandedAnimation3 * 0.1f;
-				_root->DrawTexture(*item.Item.Description.BackgroundImage, center.X, center.Y, IMenuContainer::BackgroundLayer, Alignment::Center, Vector2f(backgroundSize.X, backgroundSize.Y), Colorf(alpha, alpha, alpha, 1.0f));
+				_root->DrawTexture(*item.Item.Description.BackgroundImage, center.X, center.Y + transitionOffset, IMenuContainer::BackgroundLayer, Alignment::Center, Vector2f(backgroundSize.X, backgroundSize.Y), Colorf(alpha, alpha, alpha, transitionAlpha));
 			}
 		}
 
@@ -372,12 +424,14 @@ namespace Jazz2::UI::Menu
 		if ((episodeContinue->State.Flags & EpisodeContinuationFlags::CheatsUsed) == EpisodeContinuationFlags::CheatsUsed) {
 			levelInit.CheatsUsed = true;
 		}
+		levelInit.ElapsedMilliseconds = episodeContinue->State.ElapsedMilliseconds;
 
 		auto& firstPlayer = levelInit.PlayerCarryOvers[0];
 		firstPlayer.Lives = episodeContinue->State.Lives;
 		firstPlayer.Score = episodeContinue->State.Score;
-		std::memcpy(firstPlayer.Ammo, episodeContinue->State.Ammo, sizeof(levelInit.PlayerCarryOvers[0].Ammo));
-		std::memcpy(firstPlayer.WeaponUpgrades, episodeContinue->State.WeaponUpgrades, sizeof(levelInit.PlayerCarryOvers[0].WeaponUpgrades));
+		std::memcpy(firstPlayer.Gems, episodeContinue->State.Gems, sizeof(firstPlayer.Gems));
+		std::memcpy(firstPlayer.Ammo, episodeContinue->State.Ammo, sizeof(firstPlayer.Ammo));
+		std::memcpy(firstPlayer.WeaponUpgrades, episodeContinue->State.WeaponUpgrades, sizeof(firstPlayer.WeaponUpgrades));
 
 		_root->ChangeLevel(std::move(levelInit));
 	}
