@@ -16,7 +16,7 @@ using namespace Jazz2::UI::Menu::Resources;
 namespace Jazz2::UI::Menu
 {
 	InGameMenu::InGameMenu(LevelHandler* root)
-		: _root(root), _pressedActions(0), _touchButtonsTimer(0.0f)
+		: _root(root), _pressedActions(0), _lastNavigationFlags(NavigationFlags::AllowAll), _touchButtonsTimer(0.0f)
 	{
 		_canvasBackground = std::make_unique<MenuBackgroundCanvas>(this);
 		_canvasClipped = std::make_unique<MenuClippedCanvas>(this);
@@ -79,6 +79,22 @@ namespace Jazz2::UI::Menu
 		}
 	}
 
+	void InGameMenu::OnKeyPressed(const nCine::KeyboardEvent& event)
+	{
+		if (!_sections.empty()) {
+			auto& lastSection = _sections.back();
+			lastSection->OnKeyPressed(event);
+		}
+	}
+
+	void InGameMenu::OnKeyReleased(const nCine::KeyboardEvent& event)
+	{
+		if (!_sections.empty()) {
+			auto& lastSection = _sections.back();
+			lastSection->OnKeyReleased(event);
+		}
+	}
+
 	void InGameMenu::OnTouchEvent(const nCine::TouchEvent& event)
 	{
 		if (!_sections.empty()) {
@@ -113,7 +129,7 @@ namespace Jazz2::UI::Menu
 		std::int32_t charOffset = 0;
 		std::int32_t charOffsetShadow = 0;
 
-		float titleY = _owner->_contentBounds.Y - (ViewSize.Y >= 300 ? 30.0f : 12.0f);
+		float titleY = _owner->_contentBounds.Y - (ViewSize.Y > 330 ? 30.0f : (ViewSize.Y >= 300 ? 50.0f : 12.0f));
 		float logoBaseScale = (ViewSize.Y >= 300 ? 1.0f : 0.85f);
 		float logoScale = logoBaseScale;
 		float logoTextScale = logoBaseScale;
@@ -498,12 +514,17 @@ namespace Jazz2::UI::Menu
 			}
 		}
 
-		bool allowGamepads = true;
+		NavigationFlags flags = NavigationFlags::AllowAll;
 		if (!_sections.empty()) {
 			auto& lastSection = _sections.back();
-			allowGamepads = lastSection->IsGamepadNavigationEnabled();
+			flags = lastSection->GetNavigationFlags();
 		}
 
-		_pressedActions |= ControlScheme::FetchNativation(_root->_pressedKeys, ArrayView(joyStates, joyStatesCount), allowGamepads);
+		_pressedActions |= ControlScheme::FetchNativation(_root->_pressedKeys, ArrayView(joyStates, joyStatesCount), flags);
+		if (_lastNavigationFlags != flags) {
+			_lastNavigationFlags = flags;
+			_pressedActions &= 0xffff;
+			_pressedActions |= (_pressedActions << 16);
+		}
 	}
 }
\ No newline at end of file
