@@ -3,13 +3,10 @@
 #include "../../ContentResolver.h"
 #include "../../PreferencesCache.h"
 #include "../ControlScheme.h"
+#include "../DiscordRpcClient.h"
 #include "BeginSection.h"
 #include "FirstRunSection.h"
 
-#if (defined(DEATH_TARGET_WINDOWS) && !defined(DEATH_TARGET_WINDOWS_RT)) || defined(DEATH_TARGET_UNIX)
-#	include "../DiscordRpcClient.h"
-#endif
-
 #include "../../../nCine/Application.h"
 #include "../../../nCine/Graphics/RenderQueue.h"
 #include "../../../nCine/Audio/AudioReaderMpt.h"
@@ -24,7 +21,8 @@ namespace Jazz2::UI::Menu
 	MainMenu::MainMenu(IRootController* root, bool afterIntro)
 		: _root(root), _activeCanvas(ActiveCanvas::Background), _transitionWhite(afterIntro ? 1.0f : 0.0f),
 			_logoTransition(0.0f), _texturedBackgroundPass(this), _texturedBackgroundPhase(0.0f),
-			_pressedKeys(ValueInit, (std::size_t)KeySym::COUNT), _pressedActions(0), _touchButtonsTimer(0.0f)
+			_pressedKeys(ValueInit, (std::size_t)KeySym::COUNT), _pressedActions(0), _lastNavigationFlags(NavigationFlags::AllowAll),
+			_touchButtonsTimer(0.0f)
 	{
 		theApplication().GetGfxDevice().setWindowTitle("Jazz² Resurrection"_s);
 
@@ -97,6 +95,11 @@ namespace Jazz2::UI::Menu
 		SwitchToSection<BeginSection>();
 	}
 
+	Vector2i MainMenu::GetViewSize() const
+	{
+		return _upscalePass.GetViewSize();
+	}
+
 	void MainMenu::OnBeginFrame()
 	{
 		float timeMult = theApplication().GetTimeMult();
@@ -181,11 +184,29 @@ namespace Jazz2::UI::Menu
 	void MainMenu::OnKeyPressed(const KeyboardEvent& event)
 	{
 		_pressedKeys.set((std::size_t)event.sym);
+
+		if (!_sections.empty()) {
+			auto& lastSection = _sections.back();
+			lastSection->OnKeyPressed(event);
+		}
 	}
 
 	void MainMenu::OnKeyReleased(const KeyboardEvent& event)
 	{
 		_pressedKeys.reset((std::size_t)event.sym);
+
+		if (!_sections.empty()) {
+			auto& lastSection = _sections.back();
+			lastSection->OnKeyReleased(event);
+		}
+	}
+
+	void MainMenu::OnTextInput(const TextInputEvent& event)
+	{
+		if (!_sections.empty()) {
+			auto& lastSection = _sections.back();
+			lastSection->OnTextInput(event);
+		}
 	}
 
 	void MainMenu::OnTouchEvent(const nCine::TouchEvent& event)
@@ -215,6 +236,7 @@ namespace Jazz2::UI::Menu
 		std::int32_t charOffsetShadow = 0;
 
 		float titleY = _owner->_contentBounds.Y - (ViewSize.Y >= 300 ? 30.0f : 12.0f);
+		//titleY = _owner->_contentBounds.Y - (/*ViewSize.Y > 330 ? 30.0f :*/ (ViewSize.Y >= 300 ? 50.0f : 12.0f));
 		float logoBaseScale = (ViewSize.Y >= 300 ? 1.0f : 0.85f);
 		float logoScale = logoBaseScale + (1.0f - _owner->_logoTransition) * 7.0f;
 		float logoTextScale = logoBaseScale + (1.0f - _owner->_logoTransition) * 2.0f;
@@ -597,18 +619,23 @@ namespace Jazz2::UI::Menu
 			}
 		}
 
-		bool allowGamepads = true;
+		NavigationFlags flags = NavigationFlags::AllowAll;
 		if (!_sections.empty()) {
 			auto& lastSection = _sections.back();
-			allowGamepads = lastSection->IsGamepadNavigationEnabled();
+			flags = lastSection->GetNavigationFlags();
 		}
 
-		_pressedActions |= ControlScheme::FetchNativation(_pressedKeys, ArrayView(joyStates, joyStatesCount), allowGamepads);
+		_pressedActions |= ControlScheme::FetchNativation(_pressedKeys, ArrayView(joyStates, joyStatesCount), flags);
+		if (_lastNavigationFlags != flags) {
+			_lastNavigationFlags = flags;
+			_pressedActions &= 0xffff;
+			_pressedActions |= (_pressedActions << 16);
+		}
 	}
 
 	void MainMenu::UpdateRichPresence()
 	{
-#if (defined(DEATH_TARGET_WINDOWS) && !defined(DEATH_TARGET_WINDOWS_RT)) || defined(DEATH_TARGET_UNIX)
+#if (defined(DEATH_TARGET_WINDOWS) && !defined(DEATH_TARGET_WINDOWS_RT)) || (defined(DEATH_TARGET_UNIX) && !defined(DEATH_TARGET_SWITCH))
 		if (!PreferencesCache::EnableDiscordIntegration || !DiscordRpcClient::Get().IsSupported()) {
 			return;
 		}
