@@ -2,6 +2,7 @@
 
 #include "IMenuContainer.h"
 #include "../Canvas.h"
+#include "../ControlScheme.h"
 #include "../../../Common.h"
 
 #include "../../../nCine/Primitives/Vector2.h"
@@ -32,12 +33,15 @@ namespace Jazz2::UI::Menu
 
 		virtual void OnUpdate(float timeMult) = 0;
 		virtual void OnDraw(Canvas* canvas) = 0;
-		virtual void OnDrawClipped(Canvas* canvas) { }
-		virtual void OnDrawOverlay(Canvas* canvas) { }
+		virtual void OnDrawClipped(Canvas* canvas) {}
+		virtual void OnDrawOverlay(Canvas* canvas) {}
+		virtual void OnKeyPressed(const nCine::KeyboardEvent& event) {}
+		virtual void OnKeyReleased(const nCine::KeyboardEvent& event) {}
+		virtual void OnTextInput(const nCine::TextInputEvent& event) {}
 		virtual void OnTouchEvent(const nCine::TouchEvent& event, const Vector2i& viewSize) = 0;
 
-		virtual bool IsGamepadNavigationEnabled() const {
-			return true;
+		virtual NavigationFlags GetNavigationFlags() const {
+			return NavigationFlags::AllowAll;
 		}
 
 	protected:
