@@ -430,14 +430,16 @@ namespace Jazz2::UI::Menu
 					(previousEpisodeEnd->Flags & EpisodeContinuationFlags::IsCompleted) != EpisodeContinuationFlags::IsCompleted) {
 					levelInit.CheatsUsed = true;
 				}
+				levelInit.ElapsedMilliseconds = previousEpisodeEnd->ElapsedMilliseconds;
 
 				auto& firstPlayer = levelInit.PlayerCarryOvers[0];
 				if (previousEpisodeEnd->Lives > 0) {
 					firstPlayer.Lives = previousEpisodeEnd->Lives;
 				}
 				firstPlayer.Score = previousEpisodeEnd->Score;
-				std::memcpy(firstPlayer.Ammo, previousEpisodeEnd->Ammo, sizeof(levelInit.PlayerCarryOvers[0].Ammo));
-				std::memcpy(firstPlayer.WeaponUpgrades, previousEpisodeEnd->WeaponUpgrades, sizeof(levelInit.PlayerCarryOvers[0].WeaponUpgrades));
+				std::memcpy(firstPlayer.Gems, previousEpisodeEnd->Gems, sizeof(firstPlayer.Gems));
+				std::memcpy(firstPlayer.Ammo, previousEpisodeEnd->Ammo, sizeof(firstPlayer.Ammo));
+				std::memcpy(firstPlayer.WeaponUpgrades, previousEpisodeEnd->WeaponUpgrades, sizeof(firstPlayer.WeaponUpgrades));
 			} else {
 				// Set CheatsUsed to true if the previous episode is not completed
 				levelInit.CheatsUsed = true;
