@@ -26,8 +26,10 @@
 #include "Jazz2/PreferencesCache.h"
 #include "Jazz2/UI/Cinematics.h"
 #include "Jazz2/UI/ControlScheme.h"
+#include "Jazz2/UI/DiscordRpcClient.h"
 #include "Jazz2/UI/LoadingHandler.h"
 #include "Jazz2/UI/Menu/MainMenu.h"
+#include "Jazz2/UI/Menu/HighscoresSection.h"
 #include "Jazz2/UI/Menu/LoadingSection.h"
 #include "Jazz2/UI/Menu/SimpleMessageSection.h"
 
@@ -51,10 +53,6 @@ using namespace Jazz2::Multiplayer;
 #	include "TermLogo.h"
 #endif
 
-#if (defined(DEATH_TARGET_WINDOWS) && !defined(DEATH_TARGET_WINDOWS_RT)) || defined(DEATH_TARGET_UNIX)
-#	include "Jazz2/UI/DiscordRpcClient.h"
-#endif
-
 #if defined(DEATH_TARGET_WINDOWS) && !defined(WITH_QT5)
 #	include <cstdlib> // for `__argc` and `__argv`
 #endif
@@ -83,7 +81,7 @@ class GameEventHandler : public IAppEventHandler, public IInputEventHandler, pub
 #endif
 {
 public:
-	static constexpr std::uint16_t StateVersion = 2;
+	static constexpr std::uint16_t StateVersion = 3;
 	static constexpr char StateFileName[] = "Jazz2.resume";
 
 #if defined(WITH_MULTIPLAYER)
@@ -102,9 +100,10 @@ public:
 
 	void OnKeyPressed(const KeyboardEvent& event) override;
 	void OnKeyReleased(const KeyboardEvent& event) override;
+	void OnTextInput(const TextInputEvent& event) override;
 	void OnTouchEvent(const TouchEvent& event) override;
 
-	void InvokeAsync(Function<void()>&& callback) override;
+	void InvokeAsync(Function<void()>&& callback, const char* sourceFunc = nullptr) override;
 	void GoToMainMenu(bool afterIntro) override;
 	void ChangeLevel(LevelInitialization&& levelInit) override;
 	bool HasResumableState() const override;
@@ -151,6 +150,7 @@ private:
 	void CheckUpdates();
 #endif
 	bool SetLevelHandler(const LevelInitialization& levelInit);
+	void HandleEndOfGame(const LevelInitialization& levelInit, bool playerDied);
 	void RemoveResumableStateIfAny();
 #if defined(DEATH_TARGET_ANDROID)
 	void ApplyActivityIcon();
@@ -276,7 +276,7 @@ void GameEventHandler::OnInitialize()
 	}
 #	endif
 
-	SetStateHandler(std::make_unique<Cinematics>(this, "intro"_s, [thread](IRootController* root, bool endOfStream) mutable {
+	SetStateHandler(std::make_unique<Cinematics>(this, "intro"_s, [](IRootController* root, bool endOfStream) mutable {
 		if ((root->GetFlags() & Flags::IsVerified) == Flags::IsVerified) {
 			root->GoToMainMenu(endOfStream);
 		} else if (!endOfStream) {
@@ -292,7 +292,7 @@ void GameEventHandler::OnInitialize()
 					}
 					return false;
 				}));
-			});
+			}, NCINE_CURRENT_FUNCTION);
 		}
 
 		return true;
@@ -337,8 +337,13 @@ void GameEventHandler::OnInitialize()
 	}));
 #endif
 
+	Vector2i viewSize;
+	if (_currentHandler != nullptr) {
+		viewSize = _currentHandler->GetViewSize();
+	}
+
 	Vector2i res = theApplication().GetResolution();
-	LOGI("Rendering resolution: %ix%i", res.X, res.Y);
+	LOGI("Rendering resolution: %ix%i (%ix%i)", res.X, res.Y, viewSize.X, viewSize.Y);
 }
 
 void GameEventHandler::OnBeginFrame()
@@ -366,15 +371,17 @@ void GameEventHandler::OnResizeWindow(std::int32_t width, std::int32_t height)
 	// Resolution was changed, all viewports have to be recreated
 	Viewport::chain().clear();
 
+	Vector2i viewSize;
 	if (_currentHandler != nullptr) {
 		_currentHandler->OnInitializeViewport(width, height);
+		viewSize = _currentHandler->GetViewSize();
 	}
 
 #if defined(NCINE_HAS_WINDOWS)
 	PreferencesCache::EnableFullscreen = theApplication().GetGfxDevice().isFullscreen();
 #endif
 
-	LOGI("Rendering resolution: %ix%i", width, height);
+	LOGI("Rendering resolution: %ix%i (%ix%i)", width, height, viewSize.X, viewSize.Y);
 }
 
 void GameEventHandler::OnShutdown()
@@ -456,15 +463,20 @@ void GameEventHandler::OnKeyReleased(const KeyboardEvent& event)
 	_currentHandler->OnKeyReleased(event);
 }
 
+void GameEventHandler::OnTextInput(const TextInputEvent& event)
+{
+	_currentHandler->OnTextInput(event);
+}
+
 void GameEventHandler::OnTouchEvent(const TouchEvent& event)
 {
 	_currentHandler->OnTouchEvent(event);
 }
 
-void GameEventHandler::InvokeAsync(Function<void()>&& callback)
+void GameEventHandler::InvokeAsync(Function<void()>&& callback, const char* sourceFunc)
 {
 	_pendingCallbacks.emplace_back(std::move(callback));
-	LOGD("Callback queued for async execution");
+	LOGD("Callback queued for async execution from %s", sourceFunc);
 }
 
 void GameEventHandler::GoToMainMenu(bool afterIntro)
@@ -480,7 +492,7 @@ void GameEventHandler::GoToMainMenu(bool afterIntro)
 		} else {
 			SetStateHandler(std::make_unique<Menu::MainMenu>(this, afterIntro));
 		}
-	});
+	}, NCINE_CURRENT_FUNCTION);
 }
 
 void GameEventHandler::ChangeLevel(LevelInitialization&& levelInit)
@@ -522,7 +534,7 @@ void GameEventHandler::ChangeLevel(LevelInitialization&& levelInit)
 #endif
 				}
 			} else {
-				newHandler = std::make_unique<Menu::MainMenu>(this, false);
+				HandleEndOfGame(levelInit, false);
 #if defined(WITH_MULTIPLAYER)
 				// TODO: This should show some server console instead of exiting
 				_networkManager = nullptr;
@@ -532,10 +544,16 @@ void GameEventHandler::ChangeLevel(LevelInitialization&& levelInit)
 			// End of game
 			SaveEpisodeEnd(levelInit);
 
-			newHandler = std::make_unique<Cinematics>(this, "ending"_s, [](IRootController* root, bool endOfStream) {
-				root->GoToMainMenu(false);
+			newHandler = std::make_unique<Cinematics>(this, "ending"_s, [levelInit = std::move(levelInit)](IRootController* root, bool endOfStream) mutable {
+				auto* _this = static_cast<GameEventHandler*>(root);
+				_this->InvokeAsync([_this, levelInit = std::move(levelInit)]() {
+					_this->HandleEndOfGame(levelInit, false);
+				});
 				return true;
 			});
+		} else if (levelInit.LevelName == ":gameover"_s) {
+			// Player died
+			HandleEndOfGame(levelInit, true);
 		} else {
 			SaveEpisodeContinue(levelInit);
 
@@ -569,7 +587,7 @@ void GameEventHandler::ChangeLevel(LevelInitialization&& levelInit)
 		if (newHandler != nullptr) {
 			SetStateHandler(std::move(newHandler));
 		}
-	});
+	}, NCINE_CURRENT_FUNCTION);
 }
 
 bool GameEventHandler::HasResumableState() const
@@ -587,7 +605,7 @@ void GameEventHandler::ResumeSavedState()
 
 		auto configDir = PreferencesCache::GetDirectory();
 		auto s = fs::Open(fs::CombinePath(configDir, StateFileName), FileAccess::Read);
-		if (s->IsValid()) {
+		if (*s) {
 			std::uint64_t signature = s->ReadValue<std::uint64_t>();
 			std::uint8_t fileType = s->ReadValue<std::uint8_t>();
 			std::uint16_t version = s->ReadValue<std::uint16_t>();
@@ -603,7 +621,7 @@ void GameEventHandler::ResumeSavedState()
 
 			DeflateStream uc(*s);
 			auto levelHandler = std::make_unique<LevelHandler>(this);
-			if (levelHandler->Initialize(uc)) {
+			if (levelHandler->Initialize(uc, version)) {
 				SetStateHandler(std::move(levelHandler));
 				return;
 			}
@@ -612,7 +630,7 @@ void GameEventHandler::ResumeSavedState()
 		auto mainMenu = std::make_unique<Menu::MainMenu>(this, false);
 		mainMenu->SwitchToSection<Menu::SimpleMessageSection>(_("\f[c:#704a4a]Cannot resume saved state!\f[/c]\n\n\nMake sure all necessary files\nare accessible and try it again."), true);
 		SetStateHandler(std::move(mainMenu));
-	});
+	}, NCINE_CURRENT_FUNCTION);
 }
 
 bool GameEventHandler::SaveCurrentStateIfAny()
@@ -623,15 +641,17 @@ bool GameEventHandler::SaveCurrentStateIfAny()
 		if (levelHandler->Difficulty() != GameDifficulty::Multiplayer) {
 			auto configDir = PreferencesCache::GetDirectory();
 			auto s = fs::Open(fs::CombinePath(configDir, StateFileName), FileAccess::Write);
-			s->WriteValue<std::uint64_t>(0x2095A59FF0BFBBEF);	// Signature
-			s->WriteValue<std::uint8_t>(ContentResolver::StateFile);
-			s->WriteValue<std::uint16_t>(StateVersion);
+			if (*s) {
+				s->WriteValue<std::uint64_t>(0x2095A59FF0BFBBEF);	// Signature
+				s->WriteValue<std::uint8_t>(ContentResolver::StateFile);
+				s->WriteValue<std::uint16_t>(StateVersion);
 
-			DeflateWriter co(*s);
-			if (!levelHandler->SerializeResumableToStream(co)) {
-				LOGE("Failed to save current state");
+				DeflateWriter co(*s);
+				if (!levelHandler->SerializeResumableToStream(co)) {
+					LOGE("Failed to save current state");
+				}
+				return true;
 			}
-			return true;
 		}
 	}
 
@@ -697,7 +717,7 @@ bool GameEventHandler::CreateServer(LevelInitialization&& levelInit, std::uint16
 		auto levelHandler = std::make_unique<MultiLevelHandler>(this, _networkManager.get());
 		levelHandler->Initialize(levelInit);
 		SetStateHandler(std::move(levelHandler));
-	});
+	}, NCINE_CURRENT_FUNCTION);
 
 	return true;
 }
@@ -757,7 +777,7 @@ void GameEventHandler::OnPeerDisconnected(const Peer& peer, Reason reason)
 				case Reason::Kicked: mainMenu->SwitchToSection<Menu::SimpleMessageSection>(_("\f[c:#704a4a]Connection has been closed!\f[/c]\n\n\nYou have been \f[c:#907050]kicked\f[/c] off the server.\nContact server administrators for more information.")); break;
 				case Reason::Banned: mainMenu->SwitchToSection<Menu::SimpleMessageSection>(_("\f[c:#704a4a]Connection has been closed!\f[/c]\n\n\nYou have been \f[c:#725040]banned\f[/c] off the server.\nContact server administrators for more information.")); break;
 			}
-		});
+		}, NCINE_CURRENT_FUNCTION);
 	}
 }
 
@@ -819,7 +839,7 @@ void GameEventHandler::OnPacketReceived(const Peer& peer, std::uint8_t channelId
 					levelHandler->SetGameMode(gameMode);
 					levelHandler->Initialize(levelInit);
 					SetStateHandler(std::move(levelHandler));
-				});
+				}, NCINE_CURRENT_FUNCTION);
 				break;
 			}
 		}
@@ -1278,11 +1298,11 @@ void GameEventHandler::RefreshCacheLevels()
 		}
 #if defined(DEATH_DEBUG)
 		/*else if (extension == "j2s"_s) {
-			// Episode
+			// Translations
 			Compatibility::JJ2Strings strings;
 			strings.Open(item);
 
-			String fullPath = fs::CombinePath({ resolver.GetCachePath(), "Translations"_s, strings.Name + ".h"_s });
+			String fullPath = fs::CombinePath({ resolver.GetCachePath(), "ExtractedTranslations"_s, String(strings.Name + ".h"_s) });
 			fs::CreateDirectories(fs::GetDirectoryName(fullPath));
 			strings.Convert(fullPath, LevelTokenConversion);
 		}*/
@@ -1529,13 +1549,30 @@ bool GameEventHandler::SetLevelHandler(const LevelInitialization& levelInit)
 	return true;
 }
 
+void GameEventHandler::HandleEndOfGame(const LevelInitialization& levelInit, bool playerDied)
+{
+	const PlayerCarryOver* firstPlayer;
+	std::size_t playerCount = levelInit.GetPlayerCount(&firstPlayer);
+
+	auto mainMenu = std::make_unique<Menu::MainMenu>(this, false);
+	if (playerCount == 1 && levelInit.Difficulty != GameDifficulty::Multiplayer) {
+		std::int32_t seriesIndex = Menu::HighscoresSection::TryGetSeriesIndex(levelInit.LastEpisodeName, playerDied);
+		if (seriesIndex >= 0) {
+			mainMenu->SwitchToSection<Menu::HighscoresSection>(seriesIndex, levelInit.Difficulty,
+				levelInit.IsReforged, levelInit.CheatsUsed, levelInit.ElapsedMilliseconds, *firstPlayer);
+		}
+	}
+
+	SetStateHandler(std::move(mainMenu));
+}
+
 void GameEventHandler::WriteCacheDescriptor(const StringView path, std::uint64_t currentVersion, std::int64_t animsModified)
 {
 	auto so = fs::Open(path, FileAccess::Write);
 	so->WriteValue<std::uint64_t>(0x2095A59FF0BFBBEF);	// Signature
 	so->WriteValue<std::uint8_t>(ContentResolver::CacheIndexFile);
 	so->WriteValue<std::uint16_t>(Compatibility::JJ2Anims::CacheVersion);
-	so->WriteValue<std::uint8_t>(0x00);				// Flags
+	so->WriteValue<std::uint8_t>(0x00);					// Flags
 	so->WriteValue<std::int64_t>(animsModified);
 	so->WriteValue<std::uint16_t>((std::uint16_t)EventType::Count);
 	so->WriteValue<std::uint64_t>(currentVersion);
@@ -1547,14 +1584,8 @@ void GameEventHandler::SaveEpisodeEnd(const LevelInitialization& levelInit)
 		return;
 	}
 
-	std::size_t playerCount = 0;
-	const PlayerCarryOver* firstPlayer = nullptr;
-	for (std::size_t i = 0; i < arraySize(levelInit.PlayerCarryOvers); i++) {
-		if (levelInit.PlayerCarryOvers[i].Type != PlayerType::None) {
-			firstPlayer = &levelInit.PlayerCarryOvers[i];
-			playerCount++;
-		}
-	}
+	const PlayerCarryOver* firstPlayer;
+	std::size_t playerCount = levelInit.GetPlayerCount(&firstPlayer);
 
 	PreferencesCache::RemoveEpisodeContinue(levelInit.LastEpisodeName);
 
@@ -1580,6 +1611,8 @@ void GameEventHandler::SaveEpisodeEnd(const LevelInitialization& levelInit)
 
 			episodeEnd->Lives = firstPlayer->Lives;
 			episodeEnd->Score = firstPlayer->Score;
+			episodeEnd->ElapsedMilliseconds = levelInit.ElapsedMilliseconds;
+			std::memcpy(episodeEnd->Gems, firstPlayer->Gems, sizeof(firstPlayer->Gems));
 			std::memcpy(episodeEnd->Ammo, firstPlayer->Ammo, sizeof(firstPlayer->Ammo));
 			std::memcpy(episodeEnd->WeaponUpgrades, firstPlayer->WeaponUpgrades, sizeof(firstPlayer->WeaponUpgrades));
 		}
@@ -1601,14 +1634,8 @@ void GameEventHandler::SaveEpisodeContinue(const LevelInitialization& levelInit)
 		return;
 	}
 
-	std::size_t playerCount = 0;
-	const PlayerCarryOver* firstPlayer = nullptr;
-	for (std::size_t i = 0; i < arraySize(levelInit.PlayerCarryOvers); i++) {
-		if (levelInit.PlayerCarryOvers[i].Type != PlayerType::None) {
-			firstPlayer = &levelInit.PlayerCarryOvers[i];
-			playerCount++;
-		}
-	}
+	const PlayerCarryOver* firstPlayer;
+	std::size_t playerCount = levelInit.GetPlayerCount(&firstPlayer);
 
 	// Don't save continue in multiplayer
 	if (playerCount == 1) {
@@ -1622,6 +1649,8 @@ void GameEventHandler::SaveEpisodeContinue(const LevelInitialization& levelInit)
 		episodeContinue->State.DifficultyAndPlayerType = ((std::int32_t)levelInit.Difficulty & 0x0f) | (((std::int32_t)firstPlayer->Type & 0x0f) << 4);
 		episodeContinue->State.Lives = firstPlayer->Lives;
 		episodeContinue->State.Score = firstPlayer->Score;
+		episodeContinue->State.ElapsedMilliseconds = levelInit.ElapsedMilliseconds;
+		std::memcpy(episodeContinue->State.Gems, firstPlayer->Gems, sizeof(firstPlayer->Gems));
 		std::memcpy(episodeContinue->State.Ammo, firstPlayer->Ammo, sizeof(firstPlayer->Ammo));
 		std::memcpy(episodeContinue->State.WeaponUpgrades, firstPlayer->WeaponUpgrades, sizeof(firstPlayer->WeaponUpgrades));
 
@@ -1636,18 +1665,34 @@ void GameEventHandler::SaveEpisodeContinue(const LevelInitialization& levelInit)
 bool GameEventHandler::TryParseAddressAndPort(const StringView input, String& address, std::uint16_t& port)
 {
 	auto portSep = input.findLast(':');
-	if (portSep == nullptr) {
-		return false;
-	}
+	if (portSep) {
+		auto portString = input.suffix(portSep.begin() + 1);
+		if (portString.contains(']')) {
+			// Probably only IPv6 address (or some garbage)
+			address = input;
+			port = 0;
+			return true;
+		} else {
+			// Address (or hostname) and port
+			address = input.prefix(portSep.begin());
+			if (address.empty()) {
+				return false;
+			}
 
-	address = String(input.prefix(portSep.begin()));
-	if (address.empty()) {
-		return false;
-	}
+			auto portString = input.suffix(portSep.begin() + 1);
+			port = (std::uint16_t)stou32(portString.data(), portString.size());
+			return true;
+		}
+	} else {
+		// Address (or hostname) only
+		if (input.empty()) {
+			return false;
+		}
 
-	auto portString = input.suffix(portSep.begin() + 1);
-	port = (std::uint16_t)stou32(portString.data(), portString.size());
-	return true;
+		address = input;
+		port = 0;
+		return true;
+	}
 }
 
 void GameEventHandler::ExtractPakFile(const StringView pakFile, const StringView targetPath)
@@ -1742,8 +1787,9 @@ int PrintVersion(bool logoVisible)
 
 int main(int argc, char** argv)
 {
+#if !defined(DEATH_TARGET_SWITCH)
 	bool logoVisible = false;
-#if defined(DEATH_TRACE) && (defined(DEATH_TARGET_APPLE) || defined(DEATH_TARGET_UNIX))
+#	if defined(DEATH_TRACE) && (defined(DEATH_TARGET_APPLE) || defined(DEATH_TARGET_UNIX))
 	bool hasVirtualTerminal = ::isatty(1);
 	if (hasVirtualTerminal) {
 		const char* term = ::getenv("TERM");
@@ -1753,14 +1799,15 @@ int main(int argc, char** argv)
 			logoVisible = true;
 		}
 	}
-#endif
-#if defined(DEATH_TARGET_UNIX)
+#	endif
+#	if defined(DEATH_TARGET_UNIX)
 	for (std::size_t i = 1; i < argc; i++) {
 		if (strcmp(argv[i], "--version") == 0) {
 			// Just print current version below the logo and quit
 			return PrintVersion(logoVisible);
 		}
 	}
+#	endif
 #endif
 
 	return MainApplication::Run([]() -> std::unique_ptr<IAppEventHandler> {
