@@ -133,6 +133,11 @@ static const char ColorDimString[] = "\x1B[0;38;2;177;150;132m";
 #else
 #	include <IO/FileStream.h>
 static std::unique_ptr<Death::IO::Stream> __logFile;
+
+#	if !defined(DEATH_TRACE_ASYNC)
+#		include <Threading/Spinlock.h>
+static Death::Threading::Spinlock __logFileLock;
+#	endif
 #endif
 
 enum class ConsoleType {
@@ -145,7 +150,9 @@ enum class ConsoleType {
 };
 
 static ConsoleType __consoleType = ConsoleType::None;
+#if !defined(DEATH_TARGET_ANDROID) && !defined(DEATH_TARGET_SWITCH) && !defined(DEATH_TARGET_WINDOWS_RT)
 static bool __consoleDarkMode = true;
+#endif
 
 #if defined(DEATH_TARGET_WINDOWS) && !defined(DEATH_TARGET_WINDOWS_RT)
 #	if !defined(ENABLE_VIRTUAL_TERMINAL_PROCESSING)
@@ -170,6 +177,7 @@ static bool EnableVirtualTerminalProcessing(HANDLE consoleHandleOut)
 }
 #elif (defined(DEATH_TARGET_APPLE) || defined(DEATH_TARGET_UNIX)) && !defined(DEATH_TARGET_SWITCH)
 #	include <termios.h>
+#	include <sys/select.h>
 
 static void CheckConsoleDarkMode()
 {
@@ -280,8 +288,9 @@ namespace nCine
 				logMsgFuncLength = -1;
 				break;
 			}
-			if (message[logMsgFuncLength] == '#' && message[logMsgFuncLength + 1] == '>') {
-				logMsgFuncLength += 2;
+
+			if (std::memcmp(&message[logMsgFuncLength], " ‡ ", sizeof(" ‡ ") - 1) == 0) {
+				logMsgFuncLength += (std::int32_t)sizeof(" ‡ ") - 1;
 				break;
 			}
 			logMsgFuncLength++;
@@ -912,7 +921,14 @@ namespace nCine
 			AppendPart(logEntryWithColors, length3, message.data(), (std::int32_t)message.size());
 			logEntryWithColors[length3++] = '\n';
 
+#	if !defined(DEATH_TRACE_ASYNC)
+			// File needs to be locked, because messages can arrive from different threads
+			__logFileLock.lock();
 			__logFile->Write(logEntryWithColors, length3);
+			__logFileLock.unlock();
+#	else
+			__logFile->Write(logEntryWithColors, length3);
+#	endif
 		}
 #endif
 
@@ -1014,13 +1030,17 @@ namespace nCine
 #endif
 
 #if defined(DEATH_TRACE) && !defined(DEATH_TARGET_EMSCRIPTEN)
-		if (__logFile = fs::Open(targetPath, FileAccess::Write)) {
+		__logFile = fs::Open(targetPath, FileAccess::Write);
+		if (*__logFile) {
 #	if defined(WITH_BACKWARD)
 			// Try to save crash info to log file
-			__eh.Destination = static_cast<FileStream*>(__logFile.get())->GetHandle();
+			__eh.Destination = __logFile.get();
 #	endif
 		} else {
 			__logFile = nullptr;
+#	if defined(WITH_BACKWARD)
+			__eh.Destination = nullptr;
+#	endif
 		}
 #endif
 	}
