@@ -9,6 +9,9 @@
 #include <android/sensor.h>
 #include <cstring> // for memset()
 
+#include <Utf8.h>
+
+using namespace Death;
 using namespace Death::Containers::Literals;
 
 namespace nCine
@@ -411,21 +414,35 @@ namespace nCine
 		const int keyCode = AKeyEvent_getKeyCode(event);
 
 		// Hardware volume keys are not handled by the engine
-		if (keyCode == AKEYCODE_VOLUME_UP || keyCode == AKEYCODE_VOLUME_DOWN) {
+		if (keyCode == AKEYCODE_VOLUME_UP || keyCode == AKEYCODE_VOLUME_DOWN || keyCode == AKEYCODE_POWER) {
 			return false;
 		}
 
+		int metaState = AKeyEvent_getMetaState(event);
+
 		keyboardEvent_.scancode = AKeyEvent_getScanCode(event);
-		keyboardEvent_.sym = AndroidKeys::keySymValueToEnum(AKeyEvent_getKeyCode(event));
-		keyboardEvent_.mod = AndroidKeys::keyModMaskToEnumMask(AKeyEvent_getMetaState(event));
+		keyboardEvent_.sym = AndroidKeys::keySymValueToEnum(keyCode);
+		keyboardEvent_.mod = AndroidKeys::keyModMaskToEnumMask(metaState);
 
 		const unsigned int keySym = static_cast<unsigned int>(keyboardEvent_.sym);
-		switch (AKeyEvent_getAction(event)) {
+		const int action = AKeyEvent_getAction(event);
+		switch (action) {
 			case AKEY_EVENT_ACTION_DOWN:
 				if (keyboardEvent_.sym != KeySym::UNKNOWN) {
 					keyboardState_.keys_[keySym] = 1;
 				}
 				inputEventHandler_->OnKeyPressed(keyboardEvent_);
+
+				if ((metaState & AMETA_CTRL_ON) == 0) {
+					AndroidJniClass_KeyEvent keyEvent(AInputEvent_getType(event), keyCode);
+					if (keyEvent.isPrintingKey() || keyCode == AKEYCODE_SPACE) {
+						const int unicodeKey = keyEvent.getUnicodeChar(metaState);
+						textInputEvent_.length = Utf8::FromCodePoint(unicodeKey, textInputEvent_.text);
+						if (textInputEvent_.length > 0) {
+							inputEventHandler_->OnTextInput(textInputEvent_);
+						}
+					}
+				}
 				break;
 			case AKEY_EVENT_ACTION_UP:
 				if (keyboardEvent_.sym != KeySym::UNKNOWN) {
@@ -434,21 +451,35 @@ namespace nCine
 				inputEventHandler_->OnKeyReleased(keyboardEvent_);
 				break;
 			case AKEY_EVENT_ACTION_MULTIPLE:
-				inputEventHandler_->OnKeyPressed(keyboardEvent_);
+				// AKEY_EVENT_ACTION_MULTIPLE should be deprecated, but it seems it's still used even on Android 13
+				if (keyboardEvent_.sym != KeySym::UNKNOWN) {
+					inputEventHandler_->OnKeyPressed(keyboardEvent_);
+				}
+				// TODO: This section doesn't work anyway with software keyboards (https://stackoverflow.com/q/21124051)
+				/*else if ((metaState & AMETA_CTRL_ON) == 0) {
+					// Unicode input from software keyboard
+					long long int downTime = AKeyEvent_getDownTime(event);
+					long long int eventTime = AKeyEvent_getEventTime(event);
+					int repeatCount = AKeyEvent_getRepeatCount(event);
+					int deviceID = AInputEvent_getDeviceId(event);
+					int flags = AKeyEvent_getFlags(event);
+					int source = AInputEvent_getSource(event);
+
+					AndroidJniClass_KeyEvent keyEvent(downTime, eventTime, action, keyCode, repeatCount, metaState, deviceID, keyboardEvent_.scancode, flags, source);
+					textInputEvent_.length = keyEvent.getCharacters(textInputEvent_.text, sizeof(textInputEvent_.text));
+					if (textInputEvent_.length > 0) {
+						inputEventHandler_->OnTextInput(textInputEvent_);
+					} else if (keyEvent.isPrintingKey() || keyCode == AKEYCODE_SPACE) {
+						const int unicodeKey = keyEvent.getUnicodeChar(metaState);
+						textInputEvent_.length = Utf8::FromCodePoint(unicodeKey, textInputEvent_.text);
+						if (textInputEvent_.length > 0) {
+							inputEventHandler_->OnTextInput(textInputEvent_);
+						}
+					}
+				}*/
 				break;
 		}
 
-		// TODO: Text input
-		/*if (AKeyEvent_getAction(event) == AKEY_EVENT_ACTION_DOWN &&
-			(AKeyEvent_getMetaState(event) & AMETA_CTRL_ON) == 0) {
-			AndroidJniClass_KeyEvent keyEvent(AInputEvent_getType(event), keyCode);
-			if (keyEvent.isPrintingKey()) {
-				const int unicodeKey = keyEvent.getUnicodeChar(AKeyEvent_getMetaState(event));
-				nctl::Utf8::codePointToUtf8(unicodeKey, textInputEvent_.text, nullptr);
-				inputEventHandler_->OnTextInput(textInputEvent_);
-			}
-		}*/
-
 		return true;
 	}
 
