@@ -44,6 +44,7 @@ namespace nCine
 		{ "et", "Eesti"_s },
 		{ "fi", "Suomi"_s },
 		{ "fr", "Français"_s },
+		{ "gl", "Galego"_s },
 		{ "gr", "Greek"_s },
 		{ "hr", "Hrvatski"_s },
 		{ "hu", "Magyar"_s },
@@ -51,7 +52,9 @@ namespace nCine
 		{ "lt", "Lietuvių"_s },
 		{ "lv", "Latviešu"_s },
 		{ "mo", "ЛИМБА МОЛДОВЕНЯСКЭ"_s },
+		{ "nb", "Norsk (Bokmål)"_s },
 		{ "nl", "Nederlands"_s },
+		{ "nn", "Norsk (Nynorsk)"_s },
 		{ "no", "Norsk"_s },
 		{ "pl", "Polski"_s },
 		{ "pt", "Português"_s },
